/* 
    Argonne Leadership Computing Facility benchmark
    BlueGene/P version
    Messaging rate 
    Written by Vitali Morozov <morozov@anl.gov>
*/    
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>

#define MAXN 1000     // repetition rate for a single pair test
//#define MAXN 150000     // repetition rate for a single pair test
#define MAXSKIP 20      // skip first tests to warm-up
#define N 6             // number of simumtaneous communications (available links)

unsigned long long getticks();

main( int argc, char *argv[] )
{
    int taskid, ntasks, is1, ir1[N], i, L, k, Receiver;
    double sb, rb, d;
    int x = 13;
    MPI_Status stat[N*x];
    MPI_Request req[N*x];
    unsigned long long t1;
 
    MPI_Init( &argc, &argv );
    MPI_Comm_rank( MPI_COMM_WORLD, &taskid );
    MPI_Comm_size( MPI_COMM_WORLD, &ntasks );
   
    if ( getranks( N, &is1, &ir1[0] ) == 0 )
    {
      //fprintf( stderr,"%d: %d => %d\n", taskid, is1, ir1 );
        
        Receiver = 0;
        for ( i = 0; i < N; i++ )
            if ( taskid == ir1[i] )
                Receiver = 1; 
        
        d = 0.;

        MPI_Barrier (MPI_COMM_WORLD);

        if ( taskid == is1 )
        {
            for ( i = 0; i < N; i++ )
                fprintf( stderr,"%d => %d\n", is1, ir1[i] );

            // heating up! //
            for ( k = 0; k < MAXSKIP; k++ )
            {   
                for ( i = 0; i < N; i++ )
                    MPI_Isend( &sb, 0, MPI_BYTE, ir1[i], is1, MPI_COMM_WORLD, &req[i] );
                
                MPI_Waitall( N, req, stat );
            }
            
            t1 = getticks();
            
            for ( k = 0; k < MAXN; k++ )
            {   
                for ( i = 0; i < x*N; i++ )
                    MPI_Isend( &sb, 0, MPI_BYTE, ir1[i%6], is1, MPI_COMM_WORLD, &req[i] );
                
                MPI_Waitall( x*N, req, stat );
            }
            
            t1 = getticks() - t1;

            d = (double)(MAXN * x*N) / (double)t1 * 850.; // in million messages per second
        }

        if ( Receiver == 1 )
        {
            for ( k = 0; k < x*MAXN+MAXSKIP; k++ )
            {
                MPI_Irecv( &rb, 0, MPI_BYTE, is1, is1, MPI_COMM_WORLD, &req[0] );
                MPI_Waitall( 1, req, stat );
            }
        }

        MPI_Reduce( &d, &d, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, MPI_COMM_WORLD );
        if ( taskid == 0 )
            fprintf( stderr, "mmps: %18.8lf\n", d );
    }

    MPI_Finalize();
}
