#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <common/bgp_personality.h>
#include <common/bgp_personality_inlines.h>
#include <spi/kernel_interface.h> 

/* 
  Returns a subcommunicator which puts: 
        tasks on CPU 0 belong to one group
        tasks on CPU 1 belong to the second group
        tasks on CPU 2 belong to the third group
        tasks on CPU 3 belong to the fourth group
  Returns:
        CPUid, if split was successful
        -1, if not
*/    
int split( MPI_Comm *Split_comm )
{
  unsigned int size, rank, i, xrank, yrank, zrank, trank, myx, myy, myz, myt, myrank;
  int Nlist, *list;
  MPI_Group World_group, Split_group;

  /* Make a list of ranks */
  MPI_Comm_size( MPI_COMM_WORLD, (int *)&size );
  MPI_Comm_rank( MPI_COMM_WORLD, (int *)&myrank );
  list = malloc( sizeof( int ) * size );

  Nlist = 0;

  Kernel_Rank2Coord( myrank, &myx, &myy, &myz, &myt );

  for ( rank = 0; rank < size; rank++ )
  {
      Kernel_Rank2Coord( rank, &xrank, &yrank, &zrank, &trank );
      
      if ( trank != myt ) 
          list[ Nlist++ ] = rank;
  }

  MPI_Comm_group( MPI_COMM_WORLD, &World_group );
  MPI_Group_excl( World_group, Nlist, list, &Split_group );
  MPI_Comm_create( MPI_COMM_WORLD, Split_group, Split_comm );
  MPI_Group_free( &Split_group );

  free( list );

  return (int)myt;
}
