#include <mpi.h>
#include <stdio.h>
#include <common/bgp_personality.h>
#include <common/bgp_personality_inlines.h>
#include <spi/kernel_interface.h> 

/* Returns two tasks from the nearest nodes (1 hop distance)
   The source is always task 0 (node x,y,z, CPU t)
   The target is placed in either x+1,y,z,t or x,y+1,z,t or x,y,z+1,t
      whichever returns the taskid < size
    
   Returns:
        0, if source and target are assigned
        1, if cannot find nearest neighbor (4 tasks on 1 node is running)
*/    
int getranks( int *source, int *target )
{
  _BGP_Personality_t personality;
  _BGP_Personality_t *p = &personality;
  unsigned int x, y, z, t, size, rank;
 
  *source = 0;

  /* Determine taskid=0's node coordinates */
  Kernel_Rank2Coord( 0, &x, &y, &z, &t );

  /* check x+1 */
  Kernel_Coord2Rank( x+1, y, z, t, (unsigned *)target, &size );
  if ( (0 < *target) && (*target < size) ) return 0;

  /* check y+1 */
  Kernel_Coord2Rank( x, y+1, z, t, (unsigned *)target, &size );
  if ( (0 < *target) && (*target < size) ) return 0;
  
  /* check z+1 */
  Kernel_Coord2Rank( x, y, z+1, t, (unsigned *)target, &size );
  if ( (0 < *target) && (*target < size) ) return 0;

  return 1;
}
