#include <mpi.h>
#include <stdio.h>
#include <common/bgp_personality.h>
#include <common/bgp_personality_inlines.h>
#include <spi/kernel_interface.h> 

/* Returns two tasks with the longest path in the partition
   The source is task (0,0,0,0) in (X,Y,Z,T notation)
   The target is the task, placed to a node 
        with coordinate x-1, if X dimension is mesh
        with coordinate x/2, if X dimension is torus
        with coordinate y-1, if Y dimension is mesh
        with coordinate y/2, if Y dimension is torus
        with coordinate z-1, if Z dimension is mesh
        with coordinate z/2, if Z dimension is torus
        and t = 0, 1, 2, or 3, whichchever is available first
        
   Returns:
        0, if source and target are assigned
        1, if nothing runs on node (0,0,0)
        2, if nothing runs on the target node
*/    
int getranks( int *source, int *target )
{
  _BGP_Personality_t personality;
  _BGP_Personality_t *p = &personality;
  unsigned int x, y, z, t, size, rank;

  /* assign the taskid to the source (0,0,0,t) */
  for ( t = 0;; t++ )
  {
      Kernel_Coord2Rank( 0, 0, 0, t, (unsigned *)source, &size );
      if ( (*source >= 0) && (*source < size) && (t < 4) ) 
          break;
      else
          return 1;
  }

  /* determine the target node */
  Kernel_GetPersonality(p, sizeof(personality));

  x = BGP_Personality_xSize( p );
  y = BGP_Personality_ySize( p );
  z = BGP_Personality_zSize( p );

  if ( BGP_Personality_isTorusX( p ) ) x = x / 2; else x--;
  if ( BGP_Personality_isTorusY( p ) ) y = y / 2; else y--;
  if ( BGP_Personality_isTorusZ( p ) ) z = z / 2; else z--;

  /* assign the taskid to the target node */
  for ( t = 0;; t++ )
  {
      Kernel_Coord2Rank( x, y, z, t, (unsigned *)target, &size );
      if ( (*target >= 0) && (*target < size) && (t < 4) ) 
          break;
      else
          return 2;
  }

  return 0;
}
