/* 
    Argonne Leadership Computing Facility benchmark
    BlueGene/P version
    Partition ping-pong
    Written by Vitali Morozov <morozov@anl.gov>

    Measures the partition's longest path ping-pong
*/    
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>

#define MAXL 3          // number of message sizes 
#define MAXN 150000     // repetition rate for a single pair test
#define MAXSKIP 20      // skip first tests to warm-up

unsigned long long getticks();

main( int argc, char *argv[] )
{
    int taskid, ntasks, LENGTH[MAXL], is1, ir1, i, L, k;
    double *sb, *rb, dmin, dmax, dsum;
    MPI_Status stat;
    unsigned long long t1, tmin, tmax, tsum;
 
    LENGTH[0] = 0;
    LENGTH[1] = 4096;
    LENGTH[2] = 65536;

    posix_memalign( (void **)&sb, 16, sizeof( char ) * LENGTH[ MAXL-1 ] );
    posix_memalign( (void **)&rb, 16, sizeof( char ) * LENGTH[ MAXL-1 ] );

    MPI_Init( &argc, &argv );
    MPI_Comm_rank( MPI_COMM_WORLD, &taskid );
    MPI_Comm_size( MPI_COMM_WORLD, &ntasks );
   
    if ( getranks( &is1, &ir1 ) == 0 )
    {
        fprintf( stderr,"%d: %d => %d\n", taskid, is1, ir1 );
        
        for ( i = 0; i < MAXL; i++ )
        {
            L = LENGTH[ i ];
            tmin = 1000000000000000; 
            tmax = 0;
            tsum = 0;
            
            MPI_Barrier (MPI_COMM_WORLD);

            if ( taskid == is1 )
            {
                for ( k = 0; k < MAXN; k++ )
                {   
                    t1 = getticks();
                    MPI_Send( sb, L, MPI_BYTE, ir1, is1, MPI_COMM_WORLD );
                    MPI_Recv( rb, L, MPI_BYTE, ir1, ir1, MPI_COMM_WORLD, &stat );
                    t1 = getticks() - t1;

                    if ( k < MAXSKIP ) continue;

                    if ( tmin > t1 ) tmin = t1;
                    if ( tmax < t1 ) tmax = t1;
                    tsum += t1;
                }

                dmin = (double)tmin / 2. / 850;
                dmax = (double)tmax / 2. / 850;
                dsum = (double)tsum / (MAXN - MAXSKIP) / 2. /850;

                fprintf( stderr, "%6d: partition min, max, ave, us  %18.8lf  %18.8lf  %18.12lf\n", L, dmin, dmax, dsum );
            }

            if ( taskid == ir1 )
            {
                for ( k = 0; k < MAXN; k++ )
                {
                    MPI_Recv( rb, L, MPI_BYTE, is1, is1, MPI_COMM_WORLD, &stat );
                    MPI_Send( sb, L, MPI_BYTE, is1, ir1, MPI_COMM_WORLD );
                }
            }
        }
    }

    MPI_Finalize();
    free( sb );
    free( rb );
}
