/* 
    Argonne Leadership Computing Facility benchmark
    BlueGene/P version
    Aggregate bandwidth 
    Written by Vitali Morozov <morozov@anl.gov>
*/    
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>

#define MAXN 150000     // repetition rate for a single pair test
#define MAXSKIP 20      // skip first tests to warm-up
#define LENGTH 524288   // 512KB
#define N 6             // number of simultaneous communications (available links)

unsigned long long getticks();

main( int argc, char *argv[] )
{
    int taskid, ntasks, is1, ir1[N], i, L, k, Receiver;
    double *sb, *rb, d;
    MPI_Status stat[2*N];
    MPI_Request req[2*N];
    unsigned long long t1;
 
    MPI_Init( &argc, &argv );
    MPI_Comm_rank( MPI_COMM_WORLD, &taskid );
    MPI_Comm_size( MPI_COMM_WORLD, &ntasks );
   
    if ( getranks( N, &is1, &ir1[0] ) == 0 )
    {
        //fprintf( stderr,"%d: %d => %d\n", taskid, is1, ir1 );
        
        Receiver = 0;
        for ( i = 0; i < N; i++ )
            if ( taskid == ir1[i] )
                Receiver = 1; 
        
        d = 0.;

        posix_memalign( (void **)&sb, 16, sizeof( char ) * LENGTH );    
        posix_memalign( (void **)&rb, 16, sizeof( char ) * LENGTH );    

        MPI_Barrier (MPI_COMM_WORLD);

        if ( taskid == is1 )
        {
            for ( i = 0; i < N; i++ )
                fprintf( stderr,"%d => %d\n", is1, ir1[i] );

            // heating up! //
            for ( k = 0; k < MAXSKIP; k++ )
            {   
                for ( i = 0; i < N; i++ )
                {
                    MPI_Isend( sb, LENGTH, MPI_BYTE, ir1[i], is1,    MPI_COMM_WORLD, &req[i] );
                    MPI_Irecv( rb, LENGTH, MPI_BYTE, ir1[i], ir1[i], MPI_COMM_WORLD, &req[i+N] );
                }
                
                MPI_Waitall( 2*N, req, stat );
            }
            
            t1 = getticks();
            
            for ( k = 0; k < MAXN; k++ )
            {   
                for ( i = 0; i < N; i++ )
                {
                    MPI_Isend( sb, LENGTH, MPI_BYTE, ir1[i], is1,    MPI_COMM_WORLD, &req[i] );
                    MPI_Irecv( rb, LENGTH, MPI_BYTE, ir1[i], ir1[i], MPI_COMM_WORLD, &req[i+N] );
                }
                
                MPI_Waitall( 2*N, req, stat );
            }
            
            t1 = getticks() - t1;


            d = ( ( 850. * 2. * (double)LENGTH * (double)N * (double)MAXN ) / ( (double)t1) ); // in MB/s, 1MB = 1e6 B 

            //d = (double)(2 * MAXN * N * LENGTH) / (double)t1 * 850. / 1e6; // in MB/s, 1MB = 1e6 B
        }

        if ( Receiver == 1 )
        {
            for ( k = 0; k < MAXN+MAXSKIP; k++ )
            {
                MPI_Isend( sb, LENGTH, MPI_BYTE, is1, taskid, MPI_COMM_WORLD, &req[0] );
                MPI_Irecv( rb, LENGTH, MPI_BYTE, is1, is1,    MPI_COMM_WORLD, &req[1] );
                
                MPI_Waitall( 2, req, stat );
            }
        }

        MPI_Reduce( &d, &d, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, MPI_COMM_WORLD );
        if ( taskid == 0 )
            fprintf( stderr, "Aggregate, MB/s: %18.8lf\n", d );

        free( sb );
        free( rb );
    }

    MPI_Finalize();
}
