/* 
    Argonne Leadership Computing Facility benchmark
    BlueGene/P version
    Cartesian communicator halo exchange
    Not finished yet
    Written by Vitali Morozov <morozov@anl.gov>

    Measures the time for halo exchange on each node of a partition 
    The maximal time over all nodes is reported
*/    
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>

#define MAXN 15000       // repetition rate for a single pair test
#define MAXSKIP 20       // skip first tests to warm-up
#define LENGTH 524288    // message size

unsigned long long getticks();

main( int argc, char *argv[] )
{
    int taskid, ntasks, is1, ir1, i, k;
    double *sb, *rb, d;
    MPI_Status stat[2];
    MPI_Request req[2];
    unsigned long long t1;
 
    posix_memalign( (void **)&sb, 16, sizeof( char ) * LENGTH );
    posix_memalign( (void **)&rb, 16, sizeof( char ) * LENGTH );

    MPI_Init( &argc, &argv );
    MPI_Comm_rank( MPI_COMM_WORLD, &taskid );
    MPI_Comm_size( MPI_COMM_WORLD, &ntasks );
   
    if ( getranks( taskid, &is1, &ir1 ) == 0 )
    {
        fprintf( stderr,"%d: %d => %d\n", taskid, is1, ir1 );
        
        d = 0.;
        
        MPI_Barrier (MPI_COMM_WORLD);

        if ( taskid == is1 )
        {
            t1 = getticks();
            for ( k = 0; k < MAXN; k++ )
            {  
                MPI_Isend( sb, LENGTH, MPI_BYTE, ir1, is1, MPI_COMM_WORLD, &req[0] );
                MPI_Irecv( rb, LENGTH, MPI_BYTE, ir1, ir1, MPI_COMM_WORLD, &req[1] );
                MPI_Waitall( 2, req, stat );
            }
            t1 = getticks() - t1;

            d = 2. * 850. * (double)MAXN * (double)LENGTH / (double)t1; // in MB/s, 1MB = 1e6 bytes
        }

        if ( taskid == ir1 )
        {
            for ( k = 0; k < MAXN; k++ )
            {
                MPI_Isend( sb, LENGTH, MPI_BYTE, is1, ir1, MPI_COMM_WORLD, &req[0] );
                MPI_Irecv( rb, LENGTH, MPI_BYTE, is1, is1, MPI_COMM_WORLD, &req[1] );
                MPI_Waitall( 2, req, stat );
            }
        }

        MPI_Reduce( &d, &d, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, MPI_COMM_WORLD );
        
        if ( taskid == 0 )
            fprintf( stderr, "Bi-section, MB/s: %18.8lf\n", d );
    }

    MPI_Finalize();
    free( sb );
    free( rb );
}
