#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <common/bgp_personality.h>
#include <common/bgp_personality_inlines.h>
#include <spi/kernel_interface.h> 

/* 
   Returns two communicators, which cut a partition in two halves at coordinate c
   c = 0 - coordinate X
   c = 1 - coordinate Y
   c = 2 - coordinate Z
    
   Returns:
        0, if split was successful
        1, if given incorrect c
*/    
int split( int c, MPI_Comm *half0, MPI_Comm *half1 )
{
  _BGP_Personality_t personality;
  _BGP_Personality_t *p = &personality;
  unsigned int x, y, z, t, size, rank, i, xrank, yrank, zrank, trank;
  int Nlist0, *list0, Nlist1, *list1;
  MPI_Group World_group, Comm0_group, Comm1_group;

  /* determine the source node */
  Kernel_GetPersonality(p, sizeof(personality));

  x = BGP_Personality_xSize( p );
  y = BGP_Personality_ySize( p );
  z = BGP_Personality_zSize( p );

  /* Make 2 lists of ranks: list0 to exclude from half0, list1 to exclude from list 1 */
  MPI_Comm_size( MPI_COMM_WORLD, (int *)&size );
  list0 = malloc( sizeof( int ) * size );
  list1 = malloc( sizeof( int ) * size );

  Nlist0 = 0;
  Nlist1 = 0;

  switch ( c )
  {
      case ( 0 ):
          for ( rank = 0; rank < size; rank++ )
          {
              Kernel_Rank2Coord( rank, &xrank, &yrank, &zrank, &t );
              
              if ( xrank < x / 2 ) 
                  list1[ Nlist1++ ] = rank;
              else
                  list0[ Nlist0++ ] = rank;
          }
          break;
      case ( 1 ):
          for ( rank = 0; rank < size; rank++ )
          {
              Kernel_Rank2Coord( rank, &xrank, &yrank, &zrank, &t );
              
              if ( yrank < y / 2 ) 
                  list1[ Nlist1++ ] = rank;
              else
                  list0[ Nlist0++ ] = rank;
          }
          break;
      case ( 2 ):
          for ( rank = 0; rank < size; rank++ )
          {
              Kernel_Rank2Coord( rank, &xrank, &yrank, &zrank, &t );
              
              if ( xrank < x / 2 ) 
                  list1[ Nlist1++ ] = rank;
              else
                  list0[ Nlist0++ ] = rank;
          }
          break;
     default:
          free( list0 ); 
          free( list1 ); 
          return 1;
  }    

  MPI_Comm_group( MPI_COMM_WORLD, &World_group );
  MPI_Group_excl( World_group, Nlist0, list0, &Comm0_group );
  MPI_Group_excl( World_group, Nlist1, list1, &Comm1_group );
  MPI_Comm_create( MPI_COMM_WORLD, Comm0_group, half0 );
  MPI_Comm_create( MPI_COMM_WORLD, Comm1_group, half1 );
  MPI_Group_free( &Comm0_group );
  MPI_Group_free( &Comm1_group );

  free( list0 );
  free( list1 );
  return 0;




/*
  
*/

  return 0;
}
